from invenio_records_resources.services.records.schema import BaseRecordSchema as InvenioBaseRecordSchema

{% for b in python.record_schema_bases %}
    from {{ b|package_name }} import {{ b|base_name }}
{% endfor %}

{% include "imports" %}
