from invenio_records.dumpers import ElasticsearchDumper as InvenioElasticsearchDumper


{% for b in python.record_dumper_bases %}
from {{ b|package_name }} import {{ b|base_name }}
{% endfor %}

class {{ python.record_dumper_class|base_name }}({% for b in python.record_dumper_bases %}{{ b|base_name }}, {% endfor %}InvenioElasticsearchDumper):
    """{{ python.record_class|base_name }} elasticsearch dumper."""
